<?php if ( ! defined( 'ABSPATH' ) || ! class_exists( 'NF_Abstracts_ActionNewsletter' )) exit;
use NinjaForms\ActiveCampaign\Admin\HandleProcessData;
/**
 * Class NF_NFACDS_Actions_NFACDSAction
 */
class NF_NFACDS_Actions_NFACDSAction extends NF_Abstracts_ActionNewsletter
{
    /**
     * @var string
     */
    protected $_name  = 'nfacds';

    /**
     * @var array
     */
    protected $_tags = array();

    /**
     * @var string
     */
    protected $_timing = 'normal';

    /**
     * @var int
     */
    protected $_priority = '10';

	/**
	 * Action-specific API credentials
	 *
	 * @var array
	 */
	protected $credentials = [
		'apiUrl' =>  '',
		'apiKey' => ''
	];
	
    /**
     * Constructor
     */
    public function __construct()
	{
		parent::__construct();
		
		$this->_nicename = __( 'ActiveCampaign', 'ninja-forms-active-campaign' );
		
		$settings = NF_NFACDS()->config( 'ActionNFACDSSettings' );
		$this->_settings =  array_merge( $this->_settings, $settings );
		
		// If we aren't in the NF builder, we don't need to get a list of AC forms.
		// get_current_screen() isn't yet available, so we have to check for is_admin() and querystrings.
		
		// Added 20201.05.19 to correct issue #22; prevents undefined index
		$page = filter_input(INPUT_GET,'page');
		
		if ( ! is_admin() || 'ninja-forms' != $page || empty( $_GET[ 'form_id' ] ) ) {
			return false;
		}	
		
		$formId = filter_input(INPUT_GET,'form_id');

		if(is_numeric($formId)){

			$this->extractCredentialsByFormId( $formId);
		}
		
		$this->setMasterCredentials();

		$forms = NF_NFACDS()->get_forms();

		$this->_settings[ 'newsletter_list_forms' ][ 'options' ] = $forms;	

		$this->initializeLists();
	}

	/**
	 * Initialize lists within action settings
	 *
	 * @return void
	 */
	protected function initializeLists( ): void
	{
		$lists = $this->retrieveLists();
		
		$key = $this->get_name() . 'newsletter_list' ;
		unset($this->_settings[$key]['options']);
		
		if(!is_array($lists)){
			$lists =[];
		}
        
		foreach ($lists as $list) {
            $this->_settings[ $key][ 'options' ][] = $list;
        }
	}


	/**
	 * Retrieve lists provided by NFACDS
	 * 
	 * Can be array or boolean
	 *
	 * @return array|bool
	 */
	protected function retrieveLists( )
	{
		$lists = NF_NFACDS()->get_lists();

		return $lists;
	}

    /*
    * PUBLIC METHODS
    */

    public function save( $action_settings )
    {
		$this->extractCredentialsFromActionSettings($action_settings);

		$this->maybeUpdatePluginSettings();
    }


	/**
	 * If plugin credentials empty, update with action credentials if present
	 *
	 * @return void
	 */
	protected function maybeUpdatePluginSettings(): void
	{
		$storedApiUrl = \Ninja_Forms()->get_setting('ninja_forms_nf_nfacds_api_url','');
		$storedApiKey = \Ninja_Forms()->get_setting('ninja_forms_nf_nfacds_api_key','');
		
		if(''===$storedApiUrl && ''!==$this->credentials['apiUrl']){
			\Ninja_Forms()->update_setting('ninja_forms_nf_nfacds_api_url',$this->credentials['apiUrl']);
		}

		if(''===$storedApiKey && ''!==$this->credentials['apiKey']){
			\Ninja_Forms()->update_setting('ninja_forms_nf_nfacds_api_key',$this->credentials['apiKey']);
		}
	}

    public function process( $action_settings, $form_id, $data )
    {			
		// provide standardized handling of submission data
		$dataHandler = new HandleProcessData($data, $this->_name);

		$contact[] = array();
			
        foreach( $action_settings as $key => $value ) {		
			
            if ( FALSE === strpos( $key, "ACfields_" ) ){
				
				//In this case the Key is not a field, but could be the listId
				
				if( $key === "newsletter_list" && !empty( $value ) ){
					
					$listId = $value;
					
					$contactKeyForListId = "p[$listId]";
					
					$contact[$contactKeyForListId] = $value;
					
					$contactKeyForActive = "status[$listId]";
					
					$contact[$contactKeyForActive] = "1";
										
				}
							
			}else{
				
				$fieldId = str_replace( 'ACfields_', '', $key );
						
				if( !empty( $value ) ){
					
					
					// "Fixed" fields and "Custom" fields are composed differently in the Json request
					// If the fieldId is a numeric ID, eg: "4", it represents a "Custom" field
					if ( (string)(int)$fieldId  == $fieldId ) {
						$contactKeyForTheField = "field[$fieldId]";
										
					}else{
						
						//Otherwise, the fieldId is not a numeric id, eg: "last_name", and it represents a "fixed" field
				
						if( "tags" === $fieldId ){
							$tagsArray = array_map( 'trim', explode( "," , $value ) );
						}

						if( "hidden_form_id" === $fieldId ){
							$fieldId = "form";
						}
						$contactKeyForTheField = $fieldId;
					}
					
					$contact[$contactKeyForTheField] = $value;
				}
								
			} 
            
			
        }
		
		$data_mapping = array();
		foreach ( $data['fields'] as $element ) {
			$value_element  = (is_array($element['value'])) ? implode(',', $element['value']) : $element['value'];
			$data_mapping[]	= array( 'type' => $element['type'] , 'value' => $value_element );
		}

		foreach ( $contact as $key => $field_value ) {
				foreach($data_mapping as $data_mapping_field){
						// the case for listcheckbox 
						if($field_value == $data_mapping_field['value'] AND $data_mapping_field['type'] == 'listcheckbox'){
							$contact[$key] = str_replace(",", "||", $data_mapping_field['value']);
						}
				}
		}
		
		// extract credentials from passed action settings
		$this->extractCredentialsFromActionSettings($action_settings);

		// set master credentials prior to making request from master
		$this->setMasterCredentials();
		$responseData = NF_NFACDS()->addNewContact( $contact );
		
        // append a single response data entry to the process data
		$dataHandler->appendResponseData($responseData);
        
        return $dataHandler->toArray();
    }
	
	protected function get_lists()
    {	
		// derive form id from AJAX request
		$formId=$this->deriveFormIdFromRequest();
		
		// extract credentials from form settings' action
		$this->extractCredentialsByFormId($formId);

		// Set master credentials prior to making request from master
		$this->setMasterCredentials();		
		$lists = NF_NFACDS()->get_lists();

		return $lists;
	}

	/**
	 * Set credentials in the master plugin instance
	 *
	 * Uses the credentials derived with this action class; if no
	 * action-specific credentials have been derived, the main instance will
	 * default to the plugin specific values as needed
	 * @return void 
	 */
	protected function setMasterCredentials()
	{
		NF_NFACDS()->setActiveCampaignUrl($this->credentials['apiUrl']);
		NF_NFACDS()->setActiveCampaignKey($this->credentials['apiKey']);
	}

	/**
	 * Extracts the from Id from the AJAX request
	 *
	 * Checks the referer, gets the query string, extracts form ID ONLY if it is
	 * numeric.  This method is called from the get_lists() request passed via
	 * AJAX.
	 * 
	 * @return string 
	 */
	protected function deriveFormIdFromRequest(): string

	{
		$return = '';

		$referer = filter_input(INPUT_SERVER, 'HTTP_REFERER');

		$parsedUrl = parse_url($referer, PHP_URL_QUERY);

		parse_str($parsedUrl, $parsedQuery);

		if (isset($parsedQuery['form_id']) && is_numeric($parsedQuery['form_id'])) {
			$return = $parsedQuery['form_id'];
		}

		return $return;
	}
	
	/**
	 * Given action settings, extracts apiKey and apiUrl
	 *
	 * Action settings are passed to action at form processing time
	 * @param array $actionSettings
	 * @return void
	 */
	protected function extractCredentialsFromActionSettings(array $actionSettings): void
	{

		if (!isset($actionSettings['api_url']) || '' === $actionSettings['api_url']) {
			$apiURL = '';
		} else {
			$apiURL = trim($actionSettings['api_url']);
		}

		if (!isset($actionSettings['api_key']) || '' === $actionSettings['api_key']) {
			$apiKey = '';
		} else {
			$apiKey = trim($actionSettings['api_key']);
		}
		
		$credentials = [
			'apiUrl' =>  $apiURL,
			'apiKey' => $apiKey
		];

		$this->credentials = $credentials;
	}

	/**
	 * Extract API Key and URL by for Id, value default to empty string
	 * 
	 * Updates credentials property with values.  Ensures that value is numeric
	 *
	 * @return array
	 */
	protected function extractCredentialsByFormId($formId): void
	{
		$credentials = [
			'apiUrl' =>  '',
			'apiKey' => ''
		];

		if (is_numeric($formId)) {
			$actions = Ninja_Forms()->form($formId)->get_actions();

			foreach ($actions as $action) {
				$type = $action->get_setting('type');
				if ('nfacds' !== $type) {
					continue;
				}

				$credentials = [
					'apiUrl' =>  $action->get_setting('api_url', ''),
					'apiKey' => $action->get_setting('api_key', '')
				];

				// break after first `nfacds` action found
				break;
			}
		}

		$this->credentials = $credentials;
	}
}
