<?php

namespace NinjaForms\ActiveCampaign\Admin;

use NinjaForms\ActiveCampaign\Entities\HandledResponse;

use NinjaForms\Includes\Entities\MetaboxOutputEntity;

class MetaboxEntityConstructor
{

    public function handle($extraValue): MetaboxOutputEntity
    {
        $array = [
            'title' => __('Active Campaign Response', 'ninja-forms-active-campaign'),
            'labelValueCollection' => $this->extractResponses($extraValue)

        ];

        $return = MetaboxOutputEntity::fromArray($array);

        return $return;
    }

    protected  function extractResponses($extraValue): array
    {
        /** @var HandledResponse $handledResponse */
        $return = [];
        if (!isset($extraValue['responseData'])) {
            return $return;
        }
        foreach ($extraValue['responseData'] as $handledResponse) {
            if (!is_a($handledResponse, 'NinjaForms\ActiveCampaign\Entities\HandledResponse')) {
                continue;
            }
            $return[] = [
                'label' => __('Context', 'ninja-forms-active-campaign'),
                'value' => $this->constructContext($handledResponse)
            ];

            $resultValue = $this->constructResult($handledResponse);
            $resultStyling = ('Success'===$resultValue)?'':'alert';
            
            $return[] = [
                'label' => __('Result', 'ninja-forms-active-campaign'),
                'value' => $resultValue,
                'styling'=>$resultStyling
            ];

            $record = $this->constructRecord($handledResponse);

            if ('' != $record) {
                $return[] = [
                    'label' => __('New Record Id', 'ninja-forms-active-campaign'),
                    'value' => $record
                ];
            }

            $errorMessages = $this->constructErrorMessages($handledResponse);

            if ('' != $errorMessages) {
                $return[] = [
                    'label' => __('Error Messages', 'ninja-forms-active-campaign'),
                    'value' => $errorMessages,
                    'styling'=>'alert'
                ];
            }
        }

        return $return;
    }

    /**
     * Construct human-readable context - remove underscores/add capitalization
     * 
     * @param HandledResponse $handledResponse
     * @return string
     */
    protected function constructContext(HandledResponse $handledResponse): string
    {

        $contextString = $handledResponse->getContext();

        $contextArray = explode('_', $contextString);

        $context = implode(
            ' ',
            array_map(function ($str) {
                return ucfirst($str);
            }, $contextArray)
        );

        return $context;
    }

    /**
     * Construct human readable result as determined by HandledResponse flags
     * 
     * @param HandledResponse $handledResponse
     * @return string
     */
    protected function constructResult(HandledResponse $handledResponse): string
    {
        $result = 'Unknown Error';

        if ($handledResponse->isSuccessful()) {
            $result = 'Success';
        } elseif ($handledResponse->isApiError()) {
            $result = 'Rejected by API';
        } elseif ($handledResponse->isWpError()) {
            $result = 'Wordpress Error';
        }

        return $result;
    }

    /**
     * Construct string output for a record id if record id is present
     *
     * @param HandledResponse $handledResponse
     * @return string
     */
    protected function constructRecord(HandledResponse $handledResponse): string
    {
        $return = '';
        $records = $handledResponse->getRecords();

        $id = '';

        if (!empty($records)) {
            $arrayKeys = array_keys($records);

            $id = $arrayKeys[0];
            $return =  $id;
        }

        return $return;
    }

    /**
     * Construct error messages from HandledResponse
     *
     * @param HandledResponse $handledResponse
     * @return void
     */
    protected function constructErrorMessages(HandledResponse $handledResponse)
    {
        $return = '';
        $errorMessages = $handledResponse->getErrorMessages();
        if (!empty($errorMessages)) {
            foreach ($errorMessages as $errorMessage) {
                $return .=  $errorMessage . '  ';
            }
        }

        return $return;
    }
}
