<?php if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Class NF_NFACDS_Admin_Settings
 */
final class NF_NFACDS_Admin_Settings
{
    public function __construct()
    {
        add_filter( 'ninja_forms_plugin_settings',                  array( $this, 'plugin_settings'             ), 10, 1 );
        add_filter( 'ninja_forms_plugin_settings_groups',           array( $this, 'plugin_settings_groups'      ), 10, 1 );
		
		// Uses the same setting (the URL) to check both values. 
        add_filter( 'ninja_forms_check_setting_ninja_forms_nf_nfacds_api_url', array( $this, 'validate_nfacds_api' ), 10, 1 );
    }

	
    public function plugin_settings( $settings )
    { 
        $settings[ 'nf_nfacds' ] = NF_NFACDS()->config( 'PluginSettings' );
        return $settings;
    }
	

    public function plugin_settings_groups( $groups )
    {
		
        $groups = array_merge( $groups, NF_NFACDS()->config( 'PluginSettingsGroups' ) );
        return $groups;
    }

	
    public function validate_nfacds_api( $setting ){
		
        $apiURL = trim( Ninja_Forms()->get_setting( 'ninja_forms_nf_nfacds_api_url' ) );
        $apiKey = trim( Ninja_Forms()->get_setting( 'ninja_forms_nf_nfacds_api_key' ) );
		
		$urlIsEmpty = ( strlen( $apiURL ) === 0 ) ? true : false;
		$keyIsEmpty = ( strlen( $apiKey ) === 0 ) ? true : false;
		
		if( $urlIsEmpty && !isset( $setting['errors']['ac_error_1'] ) ){
			$setting['errors']['ac_error_1'] = __('ActiveCampaign URL cannot be empty.', 'ninja-forms-active-campaign');
		}		
		if( $keyIsEmpty && !isset( $setting['errors']['ac_error_2'] ) ){
			$setting['errors']['ac_error_2'] = __('ActiveCampaign key cannot be empty.', 'ninja-forms-active-campaign');
		}
		
		// Returns the errors and skip if any API value is empty
		if( $urlIsEmpty || $keyIsEmpty){
			return $setting;
		}
		
		
		
		$activeCampaign = NF_NFACDS::instance();
		
		$credentialsAreValid = $activeCampaign->verifyCredentials( $apiURL, $apiKey );
		
		
        if( !$credentialsAreValid && !isset( $setting[ 'errors' ]['ac_error_3'] ) ) {
            $setting[ 'errors' ]['ac_error_3'] = __( 'The ActiveCampaign API key or URL appear to be invalid.', 'ninja-forms-active-campaign');
        }		
				
        return $setting;
    }
  

}
