<?php if (! defined('ABSPATH') ) { exit;
}
use NinjaForms\ActiveCampaign\Entities\HandledResponse;
use NinjaForms\ActiveCampaign\Admin\OutputResponseDataMetabox;
/**
 * Plugin Name: Ninja Forms - ActiveCampaign
 * Plugin URI: https://ninjaforms.com
 * Description: Integrates Ninja Forms with your ActiveCampaign account so you can add users to your contact lists when they submit a Ninja Form!
 * Version: 3.1.2
 * Author: Saturday Drive
 * Author URI: https://ninjaforms.com/?utm_source=Ninja+Forms+Plugin&utm_medium=Plugins+WP+Dashboard
 * Text Domain: ninja-forms-active-campaign
 * Release Description: Merge branch 'release-3.1.2'
 */

/*use Monolog\Logger;
use Monolog\Handler\StreamHandler;
use Monolog\Handler\FirePHPHandler;*/
require_once 'vendor/autoload.php';

if(version_compare(get_option('ninja_forms_version', '0.0.0'), '3', '<') || get_option('ninja_forms_load_deprecated', false) ) {

    //include 'deprecated/ninja-forms-active-campaign.php';

} else {
    
    /**
     * The class NF_NFACDS is the plugin startup. It hooks the basic Ninja Forms addon functionality. 
     *
     * Also, it uses the ActiveCampaign API to send and retrieve information.
     */
    final class NF_NFACDS
    {
        const VERSION = '3.1.2';
        const SLUG    = 'ninja-forms-active-campaign';
        const NAME    = 'ActiveCampaign';
        const AUTHOR  = 'Saturday Drive';
        const PREFIX  = 'NF_NFACDS';

        /**
         * @var NF_NFACDS
         */
        private static $instance;

        /**
         * Plugin Directory
         * 
         * @var string $dir
         */
        public static $dir = '';

        /**
         * Plugin URL
         *
         * @var string $url
         */
        public static $url = '';

        /**
         * Logger Class (Monolog)
         * 
         * @var Monolog\Logger $logger
         */
        //private static $logger;
        
        /**
         * ActiveCampaign API Class 
         * 
         * @var ActiveCampaign $activeCampaign
         */        
        private $activeCampaign;        
        
        /**
         * ActiveCampaign Account URL 
         * 
         * @var String $activeCampaignUrl
         */            
        private $activeCampaignUrl;
            
        /**
         * ActiveCampaign Account Key 
         * 
         * @var String $activeCampaignKey
         */            
        private $activeCampaignKey;

        /**
         * Standardized HandledResponse 
         * 
         * @var HandledResponse 
         */
        protected $handledResponse;
        
        /**
         * Getter
         *
         * @return String Returns ActiveCampaign URL
         */
        public function getActiveCampaignUrl() 
        {
			return $this->activeCampaignUrl;
        }
        
        /**
         * Setter
         *
         * @param  String $activeCampaignUrl ActiveCampaign URL
         * @return void
         */
        public function setActiveCampaignUrl( $activeCampaignUrl = '' )
        {
            if(!empty($activeCampaignUrl) ) {
                $this->activeCampaignUrl = $activeCampaignUrl;
            }
        }
        
        /**
         * Getter
         *
         * @return String Returns ActiveCampaign Key
         */        
        public function getActiveCampaignKey()
        {
               return $this->activeCampaignKey;
        }    
        
        /**
         * Setter
         *
         * @param  String $activeCampaignKey ActiveCampaign Key
         * @return void
         */    
        public function setActiveCampaignKey( $activeCampaignKey = '' )
        {
            if(!empty($activeCampaignKey) ) {
                $this->activeCampaignKey = $activeCampaignKey;
            }
        }
        
        /**
         * Getter
         *
         * @return ActiveCampaign $activeCampaign  Returns the API Class
         */            
        public function getActiveCampaign()
        {
            if(is_null($this->activeCampaign) ) {
                $this->activeCampaign = new ActiveCampaign($this->getActiveCampaignUrl(), $this->getActiveCampaignKey());
            }
               return $this->activeCampaign;
        }

        /**
         * Main Plugin Instance
         *
         * Insures that only one instance of a plugin class exists in memory at any one
         * time. Also prevents needing to define globals all over the place.
         *
         * @static
         * @static var array $instance
         * @return NF_NFACDS Highlander Instance
         */
        public static function instance()
        {
            if (!isset(self::$instance) && !( self::$instance instanceof NF_NFACDS ) ) {
                self::$instance = new NF_NFACDS();

                self::$dir = plugin_dir_path(__FILE__);

                self::$url = plugin_dir_url(__FILE__);
                
                /*
                 * Register our autoloader
                 */
                spl_autoload_register(array( self::$instance, 'autoloader' ));
                
                new NF_NFACDS_Admin_Settings();
            }
            
            return self::$instance;
        }

        /**
         * Creates an instance and hooks the basic NF addon functionality
         *
         * @return Self
         */
        public function __construct()
        {
            add_action('admin_init', array( $this, 'setup_license'));

            add_filter('ninja_forms_register_fields', array($this, 'register_fields'));

            add_filter('ninja_forms_register_actions', array($this, 'register_actions'));
            
            add_action('admin_enqueue_scripts', array($this, 'load_resources'));
            add_action('ninja_forms_loaded', array($this, 'setupAdmin'));

            add_filter('nf_react_table_extra_value_keys', array($this, 'addMetabox'));
        }

        	/**
	 * Setup Admin
	 *
	 * Setup admin classes for Ninja Forms and WordPress.
	 */
	public function setupAdmin()
	{
            $handledResponseObject = HandledResponse::class;
            $extraValueKey = 'nfacds';
            $metaboxLabel = 'Active Campaign Response';
            new OutputResponseDataMetabox($handledResponseObject,$extraValueKey,$metaboxLabel);
	}

        /**
         * Add a metabox constructor to the react.js submissions page
         *
         * @param array $metaboxHandlers
         * @return array
         */
        public function addMetabox(array $metaboxHandlers): array
        {
            $metaboxHandlers['nfacds'] = '\NinjaForms\ActiveCampaign\Admin\MetaboxEntityConstructor';
            return $metaboxHandlers;
        }

        public function register_fields( $actions )
        {
            $actions[ 'nfacds-optin' ] = new NF_NFACDS_Fields_NFACDSOptIn(); 

            return $actions;
        }

        public function register_actions($actions)
        {
            $actions[ 'nfacds' ] = new NF_NFACDS_Actions_NFACDSAction();

            return $actions;
        }
        
        public function load_resources()
        {
            wp_register_script(
                self::PREFIX . 'main-js',
                plugins_url('includes/js/main.js', __FILE__),
                array( 'jquery' ),
                self::VERSION,
                true
            );
            
            wp_register_style(
                self::PREFIX . 'main-css',
                plugins_url('includes/css/main.css', __FILE__),
                array(),
                self::VERSION,
                'all'
            );            
            
            if (is_admin() ) {
                      wp_enqueue_script(self::PREFIX . 'main-js');
                      wp_enqueue_style(self::PREFIX . 'main-css');
            }
            
        }

        public function autoloader($class_name)
        {
            if (class_exists($class_name) ) { return;
            }

            if (false === strpos($class_name, self::PREFIX) ) { return;
            }

            $class_name = str_replace(self::PREFIX, '', $class_name);
            $classes_dir = realpath(plugin_dir_path(__FILE__)) . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR;
            $class_file = str_replace('_', DIRECTORY_SEPARATOR, $class_name) . '.php';

            if (file_exists($classes_dir . $class_file)) {
                include_once $classes_dir . $class_file;
            }
        }
        
        /**
         * Template
         *
         * @param string $file_name
         * @param array  $data
         */
        public static function template( $file_name = '', array $data = array() )
        {
            if(! $file_name ) { return;
            }

            extract($data);

            include self::$dir . 'includes/Templates/' . $file_name;
        }
        
        /**
         * Config
         *
         * @param  $file_name
         * @return mixed
         */
        public static function config( $file_name )
        {
            return include self::$dir . 'includes/Config/' . $file_name . '.php';
        }

        /*
         * Required methods for all extension.
         */
        public function setup_license()
        {
            if (! class_exists('NF_Extension_Updater') ) { return;
            }

            new NF_Extension_Updater(self::NAME, self::VERSION, self::AUTHOR, __FILE__, self::SLUG);
        }
        
        public function get_lists()
        {
            
			/*$apiURL = trim( Ninja_Forms()->get_setting( 'ninja_forms_nf_nfacds_api_url' ) );
			$apiKey = trim( Ninja_Forms()->get_setting( 'ninja_forms_nf_nfacds_api_key' ) );*/
		
			$credentialsAreValid = $this->setAndVerifyCredentials();
            
            if(!$credentialsAreValid ) {
                return false;
            }
            
            if(!$credentialsAreValid ) {
                //self::$logger->info('CREDENTIALS ARE NON-VALID');
            }
                            
			$lists = $this->getActiveCampaign()->api('list/list?ids=all');
            
            if(is_object($lists) && property_exists($lists, 'error') && !empty($lists->error)) {
                     // self::$logger->info('Error 1');
            }
            if(is_string($lists) && !json_decode($lists)) {
                // self::$logger->info('Error 2');
            }
            
			$listsAsArray = json_decode(json_encode($lists), true);

			// Obtains all "Custom" fields
			$fields = $this->getFields();

			// Merge with the "Fixed" fields
			$fields = $this->getFixedFields($fields);

			$listFormated = array(
				"value" => "0",
				"label" => "- "
			);
            
               $i=0;
            foreach ( $listsAsArray as $list ){
                                
                if(isset($list['name']) ) {
                    
                    $listFormated[] = array(
                     "value" => $list['id'],
                     "label" => $list['name'],
                     "groups" => ++$i,
                     "fields" => $this->getFieldsByListId($fields, $list['id'])
                    );

                    Ninja_Forms()->update_setting('active_campaign_list_' . $list['id'], $list['name']);
                }
            }
            
			return $listFormated ;
        }

        public function get_forms()
        {
			$forms = array(
				array(
					'label' => __('Select a Form', 'ninja-forms-active-campaign'),
					'value' => '',
				),
			);            
		
			$credentialsAreValid = $this->setAndVerifyCredentials();
		
            if(!$credentialsAreValid ) {
                return false;
            }                  
                
			$ac_forms = $this->getActiveCampaign()->api('form/getforms');
		
			$acFormsAsArray = json_decode(json_encode($ac_forms), true);
            
            if (! empty($acFormsAsArray) ) {
    
                foreach ( $acFormsAsArray as $form ) {
    
                    if (! is_array($form) ) {
                        continue;
                    }
                    if ($form['sendoptin'] == 0  || ! is_array($form['lists']) ) {
                        //if ( $form['sendoptin'] == 0 || ! is_array( $form['lists'] ) || ! in_array( $list_id, $form['lists'] ) ){
                        continue;
                    }                    
                    
                    // List of IDs joinned as comma separated
                    $listIDs = implode(",", $form['lists']);
    
                    $forms[] = array(
                     'label' => $form['name'],
                        
                     //Concat the Form ID with its List IDs. It's used on the client side to filter the select when the list selector is changed.
                     'value' => $form['id'] . "-" . $listIDs,
                    );
                }
            }            
            
			return $forms;
        }
        
        private function getFixedFields( $fields )
        {
            
			$labels  = array( "Tags", "Phone" , "Email" , "Last Name" , "First Name", "Hidden Form ID" );
            
            foreach ( $labels as $label ){
                
                $fieldId = str_replace(' ', '_', strtolower($label));
                                
                $newField = array(
                 'id' => $fieldId,
                 'label' => $label,
                 'value' => "ACfields_" . $fieldId,
                 'listIds' => array ( "0" => "0" )
                );
                
                array_unshift($fields, $newField);
            }
                        
               return $fields;
        }
        
        private function getFieldsByListId( $fields, $listId )
        {
            
			$fieldsById = array();
            
            foreach( $fields as $field ){
                
                $listIds = $field["listIds"];
                                
                // Array ( [0] => 0 )  means the field is available for all lists
                if($listIds["0"] === "0" ) {
                    $fieldsById[] = $field;
                }else{
                    
                    //If not, we need to find if the field is available for this single list
                    foreach( $listIds as $value ){

                        //if(isset($listIds[$value]) && $listIds[$value] === $listId ) {
                        if($value === $listId ) {
                            $fieldsById[] = $field;
                        }
                    }
                }
            }
            
			return $fieldsById;
        }
        
        public function getFields()
        {            
            
				$fields      = array();
				$method      = 'list/field/view?ids=all';
				$fields_res  = (array) $this->getActiveCampaign()->api($method);
        
            if(!empty($fields_res) ) {

                foreach($fields_res as $field){
                
                    if(is_object($field)) {

                        $fields[] = array(
                         'id'      => $field->id,
                         'label'   => $field->title,
                         'value'   => "ACfields_" . $field->id,
                         'listIds' => $field->lists,
                        );

                    }
                        
                }

            }
               return $fields;
        }

        /**
         * Add a new contact
         *
         * @param [type] $contact
         * @param array $credentials
         * @return void
         */

        public function addNewContact( $contact )
        {
            $this->handledResponse = new HandledResponse();
            
            $this->handledResponse->setContext('ActiveCampaign_addNewContact');
            
            $this->handledResponse->setTimestamp(time());

            $credentialsAreValid = $this->setAndVerifyCredentials();

            if(!$credentialsAreValid ) {
                $this->handledResponse->setIsSuccessful(FALSE);
                $this->handledResponse->setIsApiError(TRUE);
                $this->handledResponse->setErrorMessages(array('Credentials could not be verified'));
                return $this->handledResponse;
            }
            
            try{
                
                $path = 'contact/sync';               

                $contactSync = $this->getActiveCampaign()->api($path, $contact);
                $this->extractResponseData($contactSync);
            }catch( \Exception $e ){
                
                $this->handledResponse->setIsSuccessful(FALSE);
                $this->handledResponse->setIsException(TRUE);

                $this->handledResponse->setErrorMessages(
                        array('Syncing contact failed. Error returned:',
                            $e->getMessage())
                  );
            }
           return $this->handledResponse;
        }
        
    /**
     * Extract data provided with ContactSync request
     * @param obj $contactSync
     */
    protected function extractResponseData($contactSync) {
        $this->handledResponse->setResponseBody(json_encode($contactSync));
                
        if (!(int) $contactSync->success) {
            // request failed
           $this->handledResponse->setIsApiError(TRUE);
           $this->handledResponse->setErrorMessages(array(
               'Syncing contact failed. Error returned:',
               $contactSync->error
           ));
        }else{
                   
            $recordArray = array(
                $contactSync->subscriber_id=>array(
                    'id'=>$contactSync->subscriber_id,                
                )
            );
            
            $this->handledResponse->setRecords($recordArray);
            
        }

        return $this->handledResponse;
    }

        /**
         * Set credentials by passed array, default to plugin settings
         *
         * @param array $credentials
         * @return void
         */
        private function setAndVerifyCredentials()
        {
            $this->ensureFallbackCredentials();

            $credentialsAreValid = false;

            $credentialsAreValid = $this->verifyCredentials($this->activeCampaignUrl, $this->activeCampaignKey);

            return $credentialsAreValid;
        }
            
        public function verifyCredentials( $apiURL, $apiKey )
        {
               // saves new non-empty credentials 
               $this->setActiveCampaignUrl($apiURL);
               $this->setActiveCampaignKey($apiKey);        
            
               $credentialsAreValid = false;
                        
               $response = $this->getActiveCampaignProfile();            
            if(empty($response['errors']) ) {
                $credentialsAreValid = true;
            }
            
               return $credentialsAreValid;
        }
        
        /**
         * Set default credential values if not already done explicitly
         * 
         * 
         * @return void 
         */
        protected function ensureFallbackCredentials( )
        {
            if(empty($this->activeCampaignUrl)){

                $this->activeCampaignUrl = trim(Ninja_Forms()->get_setting('ninja_forms_nf_nfacds_api_url'));
            }

            if(empty($this->activeCampaignKey)){
                $this->activeCampaignKey = trim(Ninja_Forms()->get_setting('ninja_forms_nf_nfacds_api_key'));
            }

        }


        private function getActiveCampaignProfile($response = array())
        {
    
			$method = 'account/view';
    
            try {
    
                $account = $this->getActiveCampaign()->api($method);
                $fullMethod = sprintf('<a href="%1$s" target="_blank">%1$s</a>', $this->getActiveCampaignUrl() . '/'. $method);
                
                if(is_object($account) && property_exists($account, 'error') && !empty($account->error)) {
    
                    throw new \Exception(sprintf('ActiveCampaign API Error: %s, %s', $account->error, $fullMethod));
                }
    
                if(is_string($account) && !json_decode($account)) {
    
                    throw new \Exception(sprintf('ActiveCampaign API Error: %s,%s', $account, $fullMethod));
                }
    
                $response['profile'] = (array)$account;
    
            }catch(\Exception $e) {
    
                $response['errors'] = $e->getMessage();
            }
    
			return $response;
        }
    }

    /**
     * The main function responsible for returning The Highlander Plugin
     * Instance to functions everywhere.
     *
     * Use this function like you would a global variable, except without needing
     * to declare the global.
     *
     * @return {class} Highlander Instance
     */
    function NF_NFACDS()
    {
        $instance = NF_NFACDS::instance();
        
        return $instance;
    }

    NF_NFACDS();
}
